package ac.essex.ooechs.fractals.maths;

/**
 * Mandelbrot fraction
 *
 * @author Olly Oechsle, University of Essex, Date: 22-Nov-2007
 * @version 1.0
 */
public class Mandelbrot extends Fractal {

    /**
     * Implementation of the escape time algorithm for the Mandelbrot function.
     * 
     * @param x
     * @param y
     * @param max_iterations
     * @return
     */
    public double getIterations(double x, double y, final int max_iterations) {

        double x0 = x;
        double y0 = y;

        double xtemp, ytemp;

        int iteration = 0;

        double z = (x * x) + (y * y);

        // It can be proven that as soon as |z| is more than 2, the orbit will diverge
        // R is a circle in this case, although different shapes may be used to create
        // interesting variations.
        while (z < 4 && iteration < max_iterations) {

            xtemp = (x * x) - (y * y) + x0;
            ytemp = (2 * x * y) + y0;

            x = xtemp;
            y = ytemp;

            iteration++;

            z = (x * x) + (y * y);

        }


        if (iteration < max_iterations) {
            // smooth the iteration
            return  iteration + 1 - Math.log(Math.log(z)) / Math.log(2);
        }
        return iteration;

    }

}
