package ac.essex.ooechs.fractals.swing;

import ac.essex.ooechs.fractals.maths.Fractal;
import ac.essex.ooechs.fractals.maths.Mandelbrot;
import ac.essex.ooechs.fractals.colouring.Colourer;
import ac.essex.ooechs.fractals.colouring.FlameColourer;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.*;
import java.text.DecimalFormat;

/**
 * A simple SWING GUI to display the results of our Fractal Renderings
 *
 * @author Olly Oechsle, University of Essex, Date: 22-Nov-2007
 * @version 1.0
 */
public class FractalApplication extends JFrame implements FractalListener {

    protected JLabel status;
    protected JTextField bounds;
    protected JTextField max_iterations;
    protected Fractal fractal;
    protected FractalPanel fp;
    protected JCheckBox antialiasing;

    public static void main(String[] args) {
        new FractalApplication();
    }

    public FractalApplication() {

        fractal = new Mandelbrot();
        Colourer colourer = new FlameColourer();

        fp = new FractalPanel(this, fractal, colourer);

        JPanel topPanel = new JPanel(new FlowLayout());
        topPanel.setBackground(Color.BLACK);
        bounds = new JTextField();
        bounds.setForeground(Color.WHITE);
        bounds.setBackground(Color.BLACK);
        bounds.setBorder(null);

        bounds.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    try {
                        String[] b = bounds.getText().split(",");
                        double minx = 0, maxx = 0, miny = 0, maxy = 0;
                        for (int i = 0; i < b.length; i++) {
                            double d = Double.parseDouble(b[i]);
                            switch (i) {
                                case 0: minx = d; break;
                                case 1: maxx = d; break;
                                case 2: miny = d; break;
                                case 3: maxy = d; break;
                            }
                        }

                        if (minx == maxx) return;
                        if (miny == maxy) return;

                        fractal.xmin = Math.min(minx, maxx);
                        fractal.xmax = Math.max(minx, maxx);
                        fractal.ymin = Math.min(miny, maxy);
                        fractal.ymax = Math.max(miny, maxy);

                        fp.renderAgain();
                        fractalUpdated(fractal);
                    } catch (NumberFormatException err)  {
                        fractalUpdated(fractal);
                    }
                }
            }
        });

        JLabel label1 = new JLabel("Coordinates: ");
        label1.setForeground(Color.WHITE);
        topPanel.add(label1);

        topPanel.add(bounds);

        JPanel bottomPanel = new JPanel(new FlowLayout());
        bottomPanel.setBackground(Color.BLACK);

        max_iterations = new JTextField(String.valueOf(fractal.max_iterations));
        max_iterations.setForeground(Color.WHITE);
        max_iterations.setBackground(Color.BLACK);
        max_iterations.setBorder(null);
        max_iterations.setPreferredSize(new Dimension(50, 15));

        max_iterations.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    try {
                        fractal.max_iterations = Integer.parseInt(max_iterations.getText());
                        fp.renderAgain();
                    } catch (NumberFormatException err)  {
                        max_iterations.setText(String.valueOf(fractal.max_iterations));
                    }
                }
            }
        });

        JLabel label2 = new JLabel("Max Iterations: ");
        label2.setForeground(Color.WHITE);
        bottomPanel.add(label2);
        bottomPanel.add(max_iterations);

        antialiasing = new JCheckBox("HiRes");
        antialiasing.setBackground(Color.BLACK);
        antialiasing.setForeground(Color.WHITE);
        antialiasing.setSelected(fractal.hires);
        bottomPanel.add(antialiasing);

        antialiasing.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                if (fractal.hires != antialiasing.isSelected()) {
                    fractal.hires = antialiasing.isSelected();
                    fp.renderAgain();
                }
            }
        });

        status = new JLabel("");
        status.setForeground(Color.WHITE);
        status.setBackground(Color.BLACK);
        bottomPanel.add(status);

        fractalUpdated(fractal);

        Container c = getContentPane();
        c.add(fp, BorderLayout.CENTER);
        c.add(bottomPanel, BorderLayout.SOUTH);
        c.add(topPanel, BorderLayout.NORTH);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        setTitle("VASE Lab : Mandelbrot Fractal");
        setSize(500, 500);
        setVisible(true);

    }

    protected DecimalFormat f = new DecimalFormat("0.00000");

    public void fractalUpdated(Fractal fractal) {
        bounds.setText(f.format(fractal.xmin) + "," + f.format(fractal.xmax) + "," + f.format(fractal.ymin) + "," + f.format(fractal.ymax));
    }

}
